package com.agiledeveloper;

import junit.framework.TestCase;
import static org.junit.Assert.*;
import org.junit.Test;
import org.junit.Before;

public class GameOfLifeTest {
  GameOfLife _gameOfLife;
  final boolean DEAD = false;
  final boolean LIVE = true;
  boolean[][] _land;
  boolean[][] _verticalLine = new boolean[4][4];  
  boolean[][] _horizontalLine = new boolean[4][4];  
  
  @Before
  public void setUp() {
    _gameOfLife = new GameOfLife();
    _land = new boolean[4][4];

    _verticalLine[0][1] = true;
    _verticalLine[1][1] = true;
    _verticalLine[2][1] = true;
    
    _horizontalLine[1][0] = true;
    _horizontalLine[1][1] = true;
    _horizontalLine[1][2] = true;
  }
  
@Test public void testCanary() {
    assertTrue(true);
  }
  
  @Test public void testDeadCellWithZeroNeighborsStaysDead() {
    assertEquals(DEAD, _gameOfLife.isAlive(DEAD, 0));
  }

  @Test public void testDeadCellWithOneNeighborsStaysDead() {
    assertEquals(DEAD, _gameOfLife.isAlive(DEAD, 1));
  }

  @Test public void testDeadCellWithTwoNeighborsStaysDead() {
    assertEquals(DEAD, _gameOfLife.isAlive(DEAD, 2));
  }

  @Test public void testDeadCellWithFourNeighborsStaysDead() {
    assertEquals(DEAD, _gameOfLife.isAlive(DEAD, 4));
  }

  @Test public void testDeadCellWithThreeNeighborsStaysDead() {
    assertEquals(LIVE, _gameOfLife.isAlive(DEAD, 3));
  }

  @Test public void testLiveCellWithZeroNeighborsStaysDead() {
    assertEquals(DEAD, _gameOfLife.isAlive(LIVE, 0));
  }

  @Test public void testLiveCellWithOneNeighborsStaysDead() {
    assertEquals(DEAD, _gameOfLife.isAlive(LIVE, 1));
  }

  @Test public void testLiveCellWithFourNeighborsStaysDead() {
    assertEquals(DEAD, _gameOfLife.isAlive(LIVE, 4));
  }

  @Test public void testLiveCellWithThreeNeighborsStaysAlive() {
    assertEquals(LIVE, _gameOfLife.isAlive(LIVE, 3));
  }

  @Test public void testLiveCellWithTwoNeighborsStaysAlive() {
    assertEquals(LIVE, _gameOfLife.isAlive(LIVE, 2));
  }
  
  @Test public void testZeroNeighborsForACellInDeadLand() {
    assertEquals(0, _gameOfLife.getLiveNeighborCount(_land, 1, 2));
  }

  @Test public void testOneNeighborsForACellInLonerLand() {
    _land[0][1] = true;
    assertEquals(1, _gameOfLife.getLiveNeighborCount(_land, 1, 2));
  }

  @Test public void testTwoNeighborsForACellInCoupleLand() {
    _land[0][1] = true;
    _land[2][3] = true;
    assertEquals(2, _gameOfLife.getLiveNeighborCount(_land, 1, 2));
  }

  @Test public void testTwoNeighborsForALiveCellInThreeCellsLand() {
    _land[0][1] = true;
    _land[1][2] = true;
    _land[2][3] = true;
    assertEquals(2, _gameOfLife.getLiveNeighborCount(_land, 1, 2));
  }

  @Test public void testNeighborsForTopLeftCell() {
    _land[0][1] = true;
    assertEquals(1, _gameOfLife.getLiveNeighborCount(_land, 0, 0));
  }

  @Test public void testNeighborsForTopRightCell() {
    _land[1][2] = true;
    assertEquals(1, _gameOfLife.getLiveNeighborCount(_land, 0, 3));
  }

  @Test public void testNeighborsForBottomLeftCell() {
    _land[2][1] = true;
    assertEquals(1, _gameOfLife.getLiveNeighborCount(_land, 3, 0));
  }

  @Test public void testNeighborsForBottomRightCell() {
    _land[2][3] = true;
    assertEquals(1, _gameOfLife.getLiveNeighborCount(_land, 3, 3));
  }
  
  @Test public void testAllDeadCellsStayDeadInNextGen() {
    assertEquals(_land, _gameOfLife.nextGeneration(_land));
  }

  @Test public void testOneLiveCellIsDeadInNextGen() {
    _land[2][3] = true;
    boolean[][] deadLand = new boolean[4][4];
    assertEquals(deadLand, _gameOfLife.nextGeneration(_land));
  }

  @Test public void testSevenShapedLiveCellNextGenHasFourLive() {
    _land[0][0] = true;
    _land[0][1] = true;
    _land[1][1] = true;
    
    boolean[][] nextGenCells = new boolean[4][4];
    nextGenCells[0][0] = true;
    nextGenCells[0][1] = true;
    nextGenCells[1][1] = true;
    nextGenCells[1][0] = true;
    assertEquals(nextGenCells, _gameOfLife.nextGeneration(_land));
  }
  
  @Test public void testBlock() {
    _land[1][1] = _land[2][1] = _land[2][2] = _land[1][2] = true;
    assertEquals(_land, _gameOfLife.nextGeneration(_land));
  }
  
  @Test public void testVerticalOscillator() {
    assertEquals(_horizontalLine, _gameOfLife.nextGeneration(_verticalLine));
  }

  @Test public void testHorizontalOscillator() {
    assertEquals(_verticalLine, _gameOfLife.nextGeneration(_horizontalLine));
  }
}